/*
 *
 * Copyright  2003 Angelo Rosiello - Roberto Carrozzo.
 * All Rights Reserved.
 *
 *			     A.R.C.S.
 *	         Angelo Rosiello & Roberto Carrrozzo 
 *		          Stream Cipher
 *
 *
 * 			    DISCLAIMER
 * 
 * A.R.C.S. is under copyright. 
 * Please respect it to keep out from any legal drawback. 
 * The code is not free for commercial use.
 * You must contact the authors in order to get the authorization
 * to utilize ARCS.
 * If you get the authorization to use this software in a product, 
 * Angelo Rosiello and Roberto Carrozzo should be given attributions
 * as the authors of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation provided with the package.
 * Redistribution and use in source and binary forms, with modifications,
 * are not permitted.(i.e. This code cannot simply be copied or modified
 * and put under any distribution license [including the GNU public
 * license]).
 * 
 *
 * AUTHORS: Angelo Rosiello - Roberto Carrozzo
 * DATE   : 20/07/2003
 * CONTACT: angelo@rosiello.org - roberto.carrozzo@rosiello.org
 * URL	  : http://www.rosiello.org
 *
 */


#include "library.h"
#include "ARCS.h"

#define MD5_LENGTH 16
#define BLOCK_LENGTH 2048

int ARCS( char src[], char dest[], char pwd[] )
{
	/* Variable Declaration */
	register int x, streamMd5pwd, counter	;
	int i, len, endFile			;
	unsigned char crypted[BLOCK_LENGTH]	;
	unsigned char plainBlock[BLOCK_LENGTH]	;
	char anagram[50]			;
	unsigned char MD5pwd[MD5_LENGTH+1]	;
	unsigned char MD5pwdOne[MD5_LENGTH+1]	;
	unsigned char MD5pwdTwo[MD5_LENGTH+1]	;
	unsigned char MD5temp[MD5_LENGTH*3+1]	;
	FILE *sourceFile, *destinationFile	;
	/* END of Variables Declaration */

	/* Variables Initialization */
	endFile = 0; streamMd5pwd = 0		;
	MD5pwd[MD5_LENGTH] = '\0'		;
	MD5pwdOne[MD5_LENGTH] = '\0'		;
	MD5pwdTwo[MD5_LENGTH] = '\0'		;
	MD5temp[MD5_LENGTH*3] = '\0'		;
	/* END of Variables Initalization */

	strcpy( anagram, pwd )			;			// Copying password in anagram

	/* Starting the Anagram's Generation */
	anagram[0] = pwd[strlen(pwd)-3] 	;
	anagram[1] = pwd[strlen(pwd)-1]		;
	anagram[2] = pwd[strlen(pwd)-2]		;
	anagram[strlen(anagram)-1] = pwd[2] 	;
	anagram[strlen(anagram)-2] = pwd[1]	;
	anagram[strlen(anagram)-3] = pwd[0]	;
	/* END of Anagram's Generation */

	/* Opening Source and Destination Files */
	sourceFile = fopen( src, "r" )			;	
	//assert( sourceFile > 0 )			;
	if( sourceFile < 0 || sourceFile == NULL )
	{ 
		printf( "%s does not exist!\n", src )	;
		return ( -1 )				;
	}
        destinationFile = fopen( dest, "w" )		;
        //assert( destinationFile > 0 )			;
        if( destinationFile < 0 ) return ( -1 )	;

        /* Worm's Generation */
	MDString( pwd, MD5pwd , strlen(pwd) )		;		// Getting Password's MD5 -> MD5pwd
        MDString( anagram, MD5pwdOne , strlen(anagram) );		// Getting Anagram's MD5 -> MD5pwdOne

//	strcpy( MD5pwdTwo, MD5pwd )			;		// Copying MD5pwd in MD5pwdTwo
	for( i=0; i<MD5_LENGTH; i++ )
		MD5pwdTwo[i] = MD5pwd[i]		;
	
	for( i=MD5_LENGTH; i<MD5_LENGTH*2; i++ )
		MD5temp[i] = MD5pwdTwo[i-MD5_LENGTH]	;		// MD5temp[..] = MD5pwd
	for( i=MD5_LENGTH*2; i<MD5_LENGTH*3; i++ )
		MD5temp[i] = MD5pwdOne[i-MD5_LENGTH*2]	;		// MD5temp[..] = MD5pwdOne
	MD5temp[MD5_LENGTH*3] = '\0'			;

	/***********************************************
	*            MD5temp ->  Scheme			*
	*	____16_________32___________48_____	*
	*      |          |           |            |    *
	*      |NULL      |MD5_Pwd    | MD5pwdOne  |    *
	*      |__________|___________|____________|    *
	*************************************************/


        for( counter=0; counter<MD5_LENGTH; counter++ )
    	{
    		/* Shaking */
    		/* MD5temp[x] = MDpwdOne && MD5temp[y] = MDpwdDue , x !=y && x++ && y++ */
          	if( counter % 2 == 0 )
          	    MD5temp[counter] = MD5pwdOne[counter]	;
          	else
          	    MD5temp[counter] = MD5pwdTwo[counter]	;
        }

        /*******************************************************
	*                     MD5temp -> Scheme			*
	*  _____________16__________________32_________48_____	*
	* |     x!=y && x++ && y++ 	|        |            | *
	* |MD5pwdOne[x]    MD5pwd[y] 	|MD5_Pwd | MD5pwdOne  | *
	* |____________________________ |________|____________| *
	*********************************************************/


      	MDString( MD5temp, MD5pwd , MD5_LENGTH*3 )     ;		// Getting the MD5temp's MD5 in MD5pwd


      	len = BLOCK_LENGTH	;
	do
	{
		x = 0			;
		while ( x < BLOCK_LENGTH )
		{
			plainBlock[x] = fgetc( sourceFile )	;
			if( plainBlock[x] == 255 )
				if( feof( sourceFile ))
				{
					len = x		;
					endFile = 1	;
					break		;
				}
			x = x + 1 	;
		}
		for( x=0; x<len; x++ )
		{
			crypted[x] = plainBlock[x] ^ MD5pwd[streamMd5pwd]	;
			streamMd5pwd++		;
			if( streamMd5pwd == MD5_LENGTH )		// Text Length >= MD5_LENGTH
			{
			        for( counter=0; counter<MD5_LENGTH; counter++ )
    				{
    					/* MD5temp[1/2] = MD5pwd[1/2] */
    					if( counter % 2 == 0 )
		         	    		MD5temp[counter] = MD5pwd[counter]	;
		         	    	/* MD5temp[1/2] = MD5pwd[1/2] */
		         	    	else
		         	    		MD5temp[counter] = MD5pwdOne[counter]	;
        			}
			        for( counter=0; counter<MD5_LENGTH; counter++ )
    				{
		         	    MD5pwdOne[counter] = MD5pwd[counter]	;
        			}
        			
        	       		/*******************************************************************
				 *                     MD5temp -> Scheme			    * 
				 *  ______________16___________________32________________48_______  *
				 * |     x!=y && x++ && y++	|                 |               | *
				 * | MD5pwd[x]    MD5pwdOne[y] 	| Password's Md5  | Anagram's MD5 | *
				 * |____________________________|________________ |_______________| *
				 ********************************************************************/
        			
				MDString( MD5temp, MD5pwd , MD5_LENGTH*3 )	;
				streamMd5pwd = 0				;
			}
		}
		fwrite( crypted, len, 1, destinationFile )	;

	}
	while( endFile == 0 ) 		;

	fclose( sourceFile )	 	;
	fclose( destinationFile )	;

	return 0			;
}

